  program main

    use vtype_module
    use ni3_module
    use fi_module

    implicit none

    character(extstr) :: CtlFN  = 'mknc.conf'

    character(len=extstr)            :: infn
    integer(i4b)                     :: fu
    integer(i4b)                     :: ios


    integer(i4b)                     :: nheader
    integer(i4b)                     :: km

    integer     , allocatable        :: a_Lev     ( : )
    real(dp)    , allocatable        :: a_Press   ( : )
    real(dp)    , allocatable        :: a_UwFlux  ( : )
    real(dp)    , allocatable        :: a_DwFlux  ( : )
    real(dp)    , allocatable        :: a_NetFlux ( : )
    real(dp)    , allocatable        :: a_Tendency( : )

    integer     , allocatable        :: a_LevTmp     ( : )
    real(dp)    , allocatable        :: a_PressTmp   ( : )
    real(dp)    , allocatable        :: a_UwFluxTmp  ( : )
    real(dp)    , allocatable        :: a_DwFluxTmp  ( : )
    real(dp)    , allocatable        :: a_NetFluxTmp ( : )
    real(dp)    , allocatable        :: a_TendencyTmp( : )


    character(len=extstr)            :: ncfn
    integer(i4b)                     :: ncid
    character(len=extstr)            :: mode, title, com, ref, &
         name, stdname, longname, units
    integer(i4b)         , parameter :: ndims = 2
    character(len=extstr)            :: dimname( ndims )
!!$    real(dp)                         :: fillvalue

    integer(i4b)          :: k
    integer(i4b)          :: l

    integer(i4b)          :: IDPro

    real(dp)              :: WaveNumS
    real(dp)              :: WaveNumE
    integer(i4b)          :: iWaveNum

    character(extstr)     :: Line

    namelist /file_nml/ &
      & InFN, NcFN


    ! Seiff et al.
    IDPro = 1
    ! Hourdin
!!$    IDPro = 2


    title   = 'LBLRTM result'
    infn    = '../OUTPUT_RADSUM'
    ncfn    = "out/LBLRTM_Flux_Tendency.nc"
    ref     = ""
    nheader = 5

    call fi_open( CtlFN, "read", FU )
    !
    rewind( FU )
    read( FU, nml = file_nml, iostat = ios )
    if ( ios > 0 ) then
      stop 'Namelist in file_nml is something wrong.'
    end if
    write( 6, nml = file_nml )
    !
    close( FU )



    !
    ! Preparation of input file
    !
    fu = 50
    open( fu, file = infn, status='unknown' )


    !
    ! Preparation of an output NetCDF file
    !
    mode = "new"
    call ni3_open( ncfn, mode, ncid )
    !
!    com   = "Mixing ratio of H2O and O3 are converted from mass density with the use of mean atmospheric molecular weight of 28.966e-3 kg mol-1."
!!$    call ni3_set_ga( ncid, title = title, com = com, ref = ref )
    call ni3_set_ga( ncid, title = title, ref = ref )


    iWaveNum = 1
    loop: do
      read( fu, '(a)', iostat = ios ) Line
      if ( ios /= 0 ) exit loop
      !
      read( fu, '(a)', iostat = ios ) Line
      if ( ios /= 0 ) exit loop
      !Line( 1:16)
      read( Line(17:26), * ) WaveNumS
      !Line(27:27)
      read( Line(28:36), * ) WaveNumE
      !
      read( fu, '(a)', iostat = ios ) Line
      if ( ios /= 0 ) exit loop
      read( Line(19:23), * ) km
      !
      read( fu, '(a)', iostat = ios ) Line
      if ( ios /= 0 ) exit loop
      !
      read( fu, '(a)', iostat = ios ) Line
      if ( ios /= 0 ) exit loop


      write( 6, * ) iWaveNum, km, WaveNumS, WaveNumE
      call flush( 6 )


      WaveNumS = WaveNumS * 1.0d2
      WaveNumE = WaveNumE * 1.0d2

      allocate( a_Lev     ( km ) )
      allocate( a_Press   ( km ) )
      allocate( a_UwFlux  ( km ) )
      allocate( a_DwFlux  ( km ) )
      allocate( a_NetFlux ( km ) )
      allocate( a_Tendency( km ) )


      do k = 1, km
        read( fu, * ) a_Lev( k ), a_Press( k ), a_UwFlux( k ), a_DwFlux( k ), a_NetFlux( k ), a_Tendency( k )
      end do


      a_Press    = a_Press    * 1.0d2
      a_UwFlux   = a_UwFlux   * 1.0d-2
      a_DwFlux   = a_DwFlux   * 1.0d-2
      a_NetFlux  = a_NetFlux  * 1.0d-2
      a_Tendency = a_Tendency * 1.0d-2 / 86400.0d0


      allocate( a_LevTmp     ( km ) )
      allocate( a_PressTmp   ( km ) )
      allocate( a_UwFluxTmp  ( km ) )
      allocate( a_DwFluxTmp  ( km ) )
      allocate( a_NetFluxTmp ( km ) )
      allocate( a_TendencyTmp( km ) )

      a_LevTmp      = a_Lev
      a_PressTmp    = a_Press
      a_UwFluxTmp   = a_UwFlux
      a_DwFluxTmp   = a_DwFlux
      a_NetFluxTmp  = a_NetFlux
      a_TendencyTmp = a_Tendency
      do k = 1, km
        a_Lev     (k) = a_LevTmp     (km-k+1)
        a_Press   (k) = a_PressTmp   (km-k+1)
        a_UwFlux  (k) = a_UwFluxTmp  (km-k+1)
        a_DwFlux  (k) = a_DwFluxTmp  (km-k+1)
        a_NetFlux (k) = a_NetFluxTmp (km-k+1)
        a_Tendency(k) = a_TendencyTmp(km-k+1)
      end do

      deallocate( a_LevTmp      )
      deallocate( a_PressTmp    )
      deallocate( a_UwFluxTmp   )
      deallocate( a_DwFluxTmp   )
      deallocate( a_NetFluxTmp  )
      deallocate( a_TendencyTmp )


      if ( iWaveNum == 1 ) then
        !
        name     = "Press"
        stdname  = "Press"
        longname = 'pressure'
        units    = "Pa"
        call ni3_set_dim( ncid, name, NI3_REAL, a_Press, &
          stdname = stdname, longname = longname, units  = units )
        call ni3_put_att( ncid, name, 'positive', 'down' )
        !
        name     = "WaveNum"
        stdname  = "WaveNum"
        longname = 'wavenumber'
        units    = "m-1"
        call ni3_def_dim( ncid, name, NI3_REAL, NI3_UNLIMITED, &
          stdname = stdname, longname = longname, units  = units )
        !
!!$        name         = "lev"
!!$        stdname      = "lev"
!!$        longname     = stdname
!!$        units        = "1"
!!$        call ni3_def_var( ncid, name, NI3_INT, ndims, dimname, &
!!$          stdname = stdname, longname = longname, units = units )
!!$        call ni3_put_var( ncid, name, a_Lev )
        !
        dimname( 1 ) = "Press"
        dimname( 2 ) = "WaveNum"
        !
        name         = "RadUwFlux"
        stdname      = "upward_flux"
        longname     = stdname
        units        = "W m-2 (m-1)-1"
        call ni3_def_var( ncid, name, NI3_REAL, ndims, dimname, &
          stdname = stdname, longname = longname, units = units )
        name         = "RadDwFlux"
        stdname      = "downward_flux"
        longname     = stdname
        units        = "W m-2 (m-1)-1"
        call ni3_def_var( ncid, name, NI3_REAL, ndims, dimname, &
          stdname = stdname, longname = longname, units = units )
        name         = "NetFlux"
        stdname      = "net_flux"
        longname     = stdname
        units        = "W m-2 (m-1)-1"
        call ni3_def_var( ncid, name, NI3_REAL, ndims, dimname, &
          stdname = stdname, longname = longname, units = units )
        name         = "DTempDt"
        stdname      = "tendency"
        longname     = stdname
        units        = "K s-1 (m-1)-1"
        call ni3_def_var( ncid, name, NI3_REAL, ndims, dimname, &
          stdname = stdname, longname = longname, units = units )

      end if

      name         = "WaveNum"
      call ni3_put_varss( ncid, name, iWaveNum, (WaveNumS+WaveNumE)/2.0d0 )
      name         = "RadUwFlux"
      call ni3_put_varss( ncid, name, iWaveNum, a_UwFlux )
      name         = "RadDwFlux"
      call ni3_put_varss( ncid, name, iWaveNum, a_DwFlux )
      name         = "NetFlux"
      call ni3_put_varss( ncid, name, iWaveNum, a_NetFlux )
      name         = "DTempDt"
      call ni3_put_varss( ncid, name, iWaveNum, a_Tendency )

      deallocate( a_Lev      )
      deallocate( a_Press    )
      deallocate( a_UwFlux   )
      deallocate( a_DwFlux   )
      deallocate( a_NetFlux  )
      deallocate( a_Tendency )

      iWaveNum = iWaveNum + 1
    end do loop


    call ni3_close( ncid )

    close( fu )


  end program main
